import deprecated


@deprecated.deprecated(version='4.0.0', reason="Please, use 'BaseException' instead")
class ExcecaoPyCEPCorreios(Exception):
    """ Excecao base da lib"""

    def __init__(self, message=''):
        super(ExcecaoPyCEPCorreios, self).__init__(message)
        self.message = message

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.message)


@deprecated.deprecated(version='4.0.0', reason="Please, use 'BaseException' instead")
class CEPInvalido(ExcecaoPyCEPCorreios):
    """ Excecao disparada quando o CEP possui tamanho incorreto
    (diferente de 8) e/ou não existe"""


@deprecated.deprecated(version='4.0.0', reason="Please, use 'BaseException' instead")
class FalhaNaConexao(ExcecaoPyCEPCorreios):
    """ Erro de conexao durante a requisicao """


@deprecated.deprecated(version='4.0.0', reason="Please, use 'BaseException' instead")
class MultiploRedirecionamento(ExcecaoPyCEPCorreios):
    """ Excecao de multiplo redirecionamento """


@deprecated.deprecated(version='4.0.0', reason="Please, use 'BaseException' instead")
class Timeout(ExcecaoPyCEPCorreios):
    """ Erro de TimeOut  """




from pycep_correios import get_address_from_cep


def test_success():

    # Realizamos a consulta de CEP
    endereco = get_address_from_cep('37.503-130')

    assert endereco['bairro'] == 'Santo Antônio'
    assert endereco['cep'] == '37503-130'
    assert endereco['cidade'] == 'Itajubá'
    assert endereco['complemento'] == 'até 214/215'
    assert endereco['logradouro'] == 'Rua Geraldino Campista'
    assert endereco['uf'] == 'MG'




class BaseException(Exception):
    """ base exception"""

    def __init__(self, message=''):
        super(BaseException, self).__init__(message)
        self.message = message

    def __repr__(self):
        return '%s(%r)' % (self.__class__.__name__, self.message)




"""
pycep_correios.client
~~~~~~~~~~~~~~~~~~~~~
Este modulo implementa o cliente para consulta de CEP da PyCEPCorreios.

:copyright: 2016-2020 por Michell Stuttgart Faria
:license: MIT, veja o arquivo LICENSE para mais detalhes.

"""
import json
import logging
import re
import warnings

import deprecated
import requests
import zeep
from requests.packages.urllib3.exceptions import InsecureRequestWarning

from . import excecoes, exceptions

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

CARACTERES_NUMERICOS = re.compile(r'[^0-9]')
NUMBERS = re.compile(r'[^0-9]')

PRODUCAO = 1
HOMOLOGACAO = 2

URL = {
    HOMOLOGACAO: 'https://apphom.correios.com.br/SigepMasterJPA/AtendeClienteService/AtendeCliente?wsdl',  # noqa: E501
    PRODUCAO: 'https://apps.correios.com.br/SigepMasterJPA/AtendeClienteService/AtendeCliente?wsdl',  # noqa: E501
}

URL_GET_ADDRESS_FROM_CEP = 'http://www.viacep.com.br/ws/{}/json'
URL_GET_CEP_FROM_ADDRESS = 'http://www.viacep.com.br/ws/{}/{}/{}/json'


@deprecated.deprecated(version='4.0.0', reason="'consultar_cep' is no longer supported and will be removed in a future release. Please, use 'get_address_from_cep' instead.")
def consultar_cep(cep, ambiente=PRODUCAO):
    """Retorna o endereço correspondente ao número de CEP informado.

    Arguments:
        cep {str} -- CEP a ser consultado.

    Keyword Arguments:
        ambiente {int} -- Indica qual será o webservice utilizado na consulta de CEP. Valor default é PRODUCAO (default: {PRODUCAO})

    Raises:
        KeyError -- Quando ambiente selecionado não existe (esperado: PRODUCAO ou HOMOLOGACAO)
        ExcecaoPyCEPCorreios -- Quando ocorre qualquer erro na consulta do CEP.

    Returns:
        dict -- Dados do endereço do CEP consultado.
    """

    if ambiente not in URL:
        raise KeyError('Ambiente inválido! Valor deve ser 1 para produção e 2 '
                       'para homologação')

    try:
        with warnings.catch_warnings():
            # Desabilitamos o warning
            warnings.simplefilter('ignore', InsecureRequestWarning)
            warnings.simplefilter('ignore', ImportWarning)
            client = zeep.Client(URL[ambiente])
            endereco = client.service.consultaCEP(formatar_cep(cep))

            return {
                'bairro': getattr(endereco, 'bairro', ''),
                'cep': getattr(endereco, 'cep', ''),
                'cidade': getattr(endereco, 'cidade', ''),
                'end': getattr(endereco, 'end', ''),
                'uf': getattr(endereco, 'uf', ''),
                'complemento2': getattr(endereco, 'complemento2', ''),
                'unidadesPostagem': getattr(endereco, 'unidadesPostagem', []),
            }

    except zeep.exceptions.Fault as e:
        raise excecoes.ExcecaoPyCEPCorreios(message=e.message)


@deprecated.deprecated(version='4.0.0', reason="'formatar_cep' is no longer supported and will be removed in a future release. Please, use 'format_cep' instead.")
def formatar_cep(cep):
    """Formata CEP, removendo qualquer caractere não numérico.

    Arguments:
        cep {str} -- CEP a ser formatado.

    Raises:
        ValueError -- Quando a string esta vazia ou não contem numeros.

    Returns:
        str -- string contendo o CEP formatado.
    """
    if not isinstance(cep, str) or not cep:
        raise ValueError('CEP deve ser uma string não vazia '
                         'contendo somente numeros')

    return CARACTERES_NUMERICOS.sub('', cep)


@deprecated.deprecated(version='4.0.0', reason="'validar_cep' is no longer supported and will be removed in a future release. Please, use 'validate_cep' instead.")
def validar_cep(cep):
    """Verifica se o CEP informado possui 8 digitos e é constituído apenas de números.

    Arguments:
        cep {str} -- CEP a ser validado.

    Returns:
        [boolean] -- True se o CEP informado é valido. Caso contrário, retorna False.
    """
    cep = formatar_cep(cep)
    return cep.isdigit() and len(cep) == 8


def get_address_from_cep(cep):
    """Retorna o endereço correspondente ao número de CEP informado.

    Arguments:
        cep {str} -- CEP a ser consultado.
    Raises:
        BaseException -- Quando ocorre qualquer erro na consulta do CEP.
    Returns:
        dict -- Dados do endereço do CEP consultado.
    """

    cep = format_cep(cep)

    try:
        response = requests.get(URL_GET_ADDRESS_FROM_CEP.format(cep))

        if response.status_code == 200:
            address = json.loads(response.text)

            if address.get('erro'):
                raise exceptions.BaseException(message='Other error')

            return {
                'bairro': address.get('bairro', ''),
                'cep': address.get('cep', ''),
                'cidade': address.get('localidade', ''),
                'logradouro': address.get('logradouro', ''),
                'uf': address.get('uf', ''),
                'complemento': address.get('complemento', ''),
            }

        elif response.status_code == 400:
            raise exceptions.BaseException(message='Invalid CEP: %s' % cep)  # noqa
        else:
            raise exceptions.BaseException(message='Other error')

    except requests.exceptions.RequestException as e:
        raise exceptions.BaseException(message=e.message)


def get_cep_from_address(state, city, street):
    """Retorna os CEPs correspondente ao endereço informado.

    Arguments:
        state {str} -- Sigla do estado da consulta
        city {str} -- Cidade do CEP ser encontrado
        street {str} -- Rua do CEP a ser encontrado
    Raises:
        BaseException -- Quando ocorre qualquer erro na consulta do CEP.
    Returns:
        dict -- Dados do endereço do CEP consultado.
    """

    try:
        response = requests.get(
            URL_GET_CEP_FROM_ADDRESS.format(state, city, street))

        if response.status_code == 200:
            return response.json()

        elif response.status_code == 400:
            raise exceptions.BaseException(
                message='City and Street must be 3 characters of lenght')
        else:
            raise exceptions.BaseException(
                message='Other error ocurred!')

    except requests.exceptions.RequestException as e:
        raise exceptions.BaseException(message=e.message)


def format_cep(cep):
    """Formata CEP, removendo qualquer caractere não numérico.

    Arguments:
        cep {str} -- CEP a ser formatado.
    Raises:
        ValueError -- Quando a string esta vazia ou não contem numeros.
    Returns:
        str -- string contendo o CEP formatado.
    """
    if not isinstance(cep, str) or not cep:
        raise ValueError(
            'CEP must be a non-empty string containing only numbers')

    return NUMBERS.sub('', cep)


def validate_cep(cep):
    """Verifica se o CEP informado possui 8 digitos e é constituído apenas de números.
    Arguments:
        cep {str} -- CEP a ser validado.
    Returns:
        [boolean] -- True se o CEP informado é valido. Caso contrário, retorna False.
    """
    cep = format_cep(cep)
    return cep.isdigit() and len(cep) == 8



from .client import (HOMOLOGACAO, PRODUCAO, consultar_cep, format_cep,  # noqa
                     formatar_cep, get_address_from_cep, get_cep_from_address,
                     validar_cep, validate_cep)
from .excecoes import CEPInvalido
from .exceptions import BaseException

__all__ = [
    'consultar_cep',
    'formatar_cep',
    'validar_cep',
    'get_address_from_cep',
    'get_cep_from_address',
    'format_cep',
    'validate_cep',
    'CEPInvalido',
    'BaseException',
    'HOMOLOGACAO',
    'PRODUCAO',
]



#!/usr/bin/env python
from codecs import open

from setuptools import setup

with open('README.md', 'r') as readme_file:
    readme = readme_file.read()

with open('requirements.txt', 'r') as requirements_file:
    requirements = requirements_file.read()

setup(
    name='pycep_correios',
    version='4.0.3',
    description='API para consulta de endereços e CEPs',
    long_description=readme,
    long_description_content_type='text/markdown',
    author='Michell Stuttgart Faria',
    author_email='michellstut@gmail.com',
    maintainer='Michell Stuttgart Faria',
    maintainer_email='michellstut@gmail.com',
    url='https://github.com/mstuttgart/pycep-correios',
    download_url='https://github.com/mstuttgart/pycep-correios',
    packages=[
        'pycep_correios',
    ],
    package_dir={
        'pycep_correios': 'pycep_correios',
    },
    include_package_data=True,
    install_requires=requirements,
    license='MIT License',
    zip_safe=False,
    keywords='correios desenvolvimento busca endereco cep',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: Portuguese',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    test_suite='tests',
)



import pytest

from pycep_correios import format_cep


def test_success():

    assert format_cep('37.503-003') == '37503003'
    assert format_cep('   37.503-003') == '37503003'
    assert format_cep('37 503-003') == '37503003'
    assert format_cep('37.503&003saasd') == '37503003'
    assert format_cep('\n \r 37.503-003') == '37503003'
    assert format_cep('\n \r 37.503-003') == '37503003'

    # ponto e virgula
    assert format_cep('37.503-003;') == '37503003'

    # Unicode Greek Question Mark
    assert format_cep(u'37.503-003;') == '37503003'


def test_fail():

    with pytest.raises(ValueError):
        format_cep(37503003)

    with pytest.raises(ValueError):
        format_cep('')

    with pytest.raises(ValueError):
        format_cep(None)

    with pytest.raises(ValueError):
        format_cep(False)

    with pytest.raises(ValueError):
        format_cep(True)



from pycep_correios import get_cep_from_address


def test_success():

    response = [
        {
            "cep": "37503-165",
            "logradouro": "Rua Geraldino Campista",
            "complemento": "de 871/872 ao fim",
            "bairro": "Santa Luzia",
            "localidade": "Itajubá",
            "uf": "MG",
            "unidade": "",
            "ibge": "3132404",
            "gia": ""
        },
        {
            "cep": "37503-130",
            "logradouro": "Rua Geraldino Campista",
            "complemento": "até 214/215",
            "bairro": "Santo Antônio",
            "localidade": "Itajubá",
            "uf": "MG",
            "unidade": "",
            "ibge": "3132404",
            "gia": ""
        },
        {
            "cep": "37503-003",
            "logradouro": "Rua Geraldino Campista",
            "complemento": "de 216/217 a 869/870",
            "bairro": "Vila Poddis",
            "localidade": "Itajubá",
            "uf": "MG",
            "unidade": "",
            "ibge": "3132404",
            "gia": ""
        }
    ]

    # Realizamos a consulta de enreço
    ceps = get_cep_from_address(
        state='MG', city='Itajuba', street='Rua Geraldino Campista')

    assert response == ceps



from unittest import TestCase, mock

import pytest

from pycep_correios import consultar_cep, excecoes, formatar_cep, validar_cep


class TestPyCEPCorreios(TestCase):

    def setUp(self):
        self.expected_address = {
            'bairro': 'Santo Antônio',
            'cep': '37503130',
            'cidade': 'Itajubá',
            'end': 'Rua Geraldino Campista',
            'uf': 'MG',
            'complemento2': '- até 214/215',
            'unidadesPostagem': [],
        }

    @mock.patch('zeep.Client')
    def test_consultar_cep(self, mk):

        class MockClass:
            def __init__(self, dictionary):
                for k, v in dictionary.items():
                    setattr(self, k, v)

        end_esperado = {
            'bairro': 'Santo Antônio',
            'cep': '37503130',
            'cidade': 'Itajubá',
            'complemento2': '- até 214/215',
            'end': 'Rua Geraldino Campista',
            'uf': 'MG',
            'unidadesPostagem': [],
        }

        service_mk = mk.return_value.service

        # Criamos o mock para o valor de retorno
        service_mk.consultaCEP.return_value = MockClass(end_esperado)

        # Realizamos a consulta de CEP
        endereco = consultar_cep('37.503-130')

        assert endereco['bairro'] == 'Santo Antônio'
        assert endereco['cep'] == '37503130'
        assert endereco['cidade'] == 'Itajubá'
        assert endereco['complemento2'] == '- até 214/215'
        assert endereco['end'] == 'Rua Geraldino Campista'
        assert endereco['uf'] == 'MG'
        assert endereco['unidadesPostagem'] == []

        # Verifica se o metodo consultaCEP foi chamado
        # com os parametros corretos
        service_mk.consultaCEP.assert_called_with('37503130')

    @mock.patch('zeep.Client')
    def test_consultar_return_error(self, mk):

        class MockClass:
            def __init__(self, dictionary):
                for k, v in dictionary.items():
                    setattr(self, k, v)

        # Utilizamps um retorno vazio para testar o retorno
        # com atributos ausentes
        end_esperado = {
        }

        service_mk = mk.return_value.service

        # Criamos o mock para o valor de retorno
        service_mk.consultaCEP.return_value = MockClass(end_esperado)

        # Realizamos a consulta de CEP
        endereco = consultar_cep('37.503-130')

        assert endereco['bairro'] == ''
        assert endereco['cep'] == ''
        assert endereco['cidade'] == ''
        assert endereco['complemento2'] == ''
        assert endereco['end'] == ''
        assert endereco['uf'] == ''
        assert endereco['unidadesPostagem'] == []

    def test_formatar_cep(self):

        with pytest.raises(ValueError):
            formatar_cep(37503003)

        with pytest.raises(ValueError):
            formatar_cep('')

        with pytest.raises(ValueError):
            formatar_cep(None)

        with pytest.raises(ValueError):
            formatar_cep(False)

        with pytest.raises(ValueError):
            formatar_cep(True)

        assert formatar_cep('37.503-003') == '37503003'
        assert formatar_cep('   37.503-003') == '37503003'
        assert formatar_cep('37 503-003') == '37503003'
        assert formatar_cep('37.503&003saasd') == '37503003'
        assert formatar_cep('\n \r 37.503-003') == '37503003'
        assert formatar_cep('\n \r 37.503-003') == '37503003'

        # ponto e virgula
        self.assertEqual(formatar_cep('37.503-003;'), '37503003')

        # Unicode Greek Question Mark
        self.assertEqual(formatar_cep(u'37.503-003;'), '37503003')

    def test_validar_cep(self):

        with pytest.raises(ValueError):
            validar_cep(37503003)

        with pytest.raises(ValueError):
            validar_cep('')

        assert validar_cep('37.503-003')
        assert validar_cep('   37.503-003')
        assert validar_cep('37.503&003saasd')

        assert not validar_cep('37.503-00')

    def test_consultar_cep_integration(self):

        # Aqui realizamos um teste de integração
        # pra garantir o funcionamento do todo

        # Realizamos a consulta de CEP
        try:
            endereco = consultar_cep('37.503-130')
        except excecoes.ExcecaoPyCEPCorreios as exc:
            print(exc.message)

        assert endereco['bairro'] == 'Santo Antônio'
        assert endereco['cep'] == '37503130'
        assert endereco['cidade'] == 'Itajubá'
        assert endereco['complemento2'] == '- até 214/215'
        assert endereco['end'] == 'Rua Geraldino Campista'
        assert endereco['uf'] == 'MG'
        assert endereco['unidadesPostagem'] == []



import pytest

from pycep_correios import validar_cep


def test_success():

    assert validar_cep('37.503-003')
    assert validar_cep('   37.503-003')
    assert validar_cep('37.503&003saasd')

    assert not validar_cep('37.503-00')


def test_fail():

    with pytest.raises(ValueError):
        validar_cep(37503003)

    with pytest.raises(ValueError):
        validar_cep('')
